package com.tuoheng.airportGzdp.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.tuoheng.airportGzdp.common.BaseServiceImpl;
import com.tuoheng.airportGzdp.common.JsonResult;
import com.tuoheng.airportGzdp.entity.AirCurrent;
import com.tuoheng.airportGzdp.entity.AirlineFile;
import com.tuoheng.airportGzdp.entity.InspectionRecord;
import com.tuoheng.airportGzdp.mapper.AirCurrentMapper;
import com.tuoheng.airportGzdp.service.AirCurrentService;
import com.tuoheng.airportGzdp.vo.AirCurrentDto;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;


/**
 * <p>
 * 航线文件表 服务实现类
 * </p>
 *
 * @author 拓恒
 * @since 2021-09-26
 */
@Service
@Slf4j
public class AirCurrentServiceImpl extends BaseServiceImpl<AirCurrentMapper, AirCurrent> implements AirCurrentService {

    @Resource
    private AirCurrentMapper airCurrentMapper;

    @Resource
    private AirlineFileServiceImpl airlineFileServiceImpl;

    @Resource
    private InspectionRecordServiceImpl iInspectionRecordService;

    /**
     * 查询 流量测速 分页数据
     * @param dto
     * @return
     */
    @Override
    public JsonResult getAirCurrentPage(AirCurrentDto dto){
        //判断开始 结束时间是否都填上了
        if (Objects.isNull(dto.getStartTime()) | Objects.isNull(dto.getEndTime()) ){
            return JsonResult.error("开始日期和节数日期为必填项！");
        }
        //查询数据
        //条件查询总数
        Integer sumNum = dto.getLimit();
        //计算从N开始
        Integer pageIndex = (dto.getPage() - 1) * dto.getLimit();
        QueryWrapper<AirCurrent> queryWrapper = new QueryWrapper<>();
        queryWrapper.ge("air_fly_time", dto.getStartTime() + " " + "00:00:00");
        queryWrapper.le("air_fly_time", dto.getEndTime() + " " + "23:59:59");
        queryWrapper.eq("mark", 1);
        //查询条数
        Integer integer = airCurrentMapper.selectCount(queryWrapper);
        queryWrapper.orderByDesc("air_fly_time");
        queryWrapper.last("LIMIT " + pageIndex + "," + dto.getLimit());
        IPage<AirCurrent> page = new Page<>(dto.getPage(), dto.getLimit());
        IPage<AirCurrent> airCurrentIPage = page(page, queryWrapper);
        //处理流速度的数据
        if (!CollectionUtils.isEmpty(airCurrentIPage.getRecords())){
            List<AirCurrent> records = airCurrentIPage.getRecords();
            airCurrentIPage.setTotal(integer);
            //查询航线名称
            Map<Integer, AirlineFile> lineIdMap = new HashMap<>();
            List<Integer> airlineFileIdList =
                    records.stream().map(o -> o.getAirlineFileId()).collect(Collectors.toList());
            if (!CollectionUtils.isEmpty(airlineFileIdList)){
                LambdaQueryWrapper<AirlineFile> airlineFileLambdaQueryWrapper = new LambdaQueryWrapper<>();
                airlineFileLambdaQueryWrapper.in(AirlineFile:: getId ,airlineFileIdList);
                List<AirlineFile> list = airlineFileServiceImpl.list(airlineFileLambdaQueryWrapper);
                lineIdMap = list.stream().collect(Collectors.toMap(AirlineFile::getId, a -> a));
            }
            Map<Integer, AirlineFile> finalLineIdMap = lineIdMap;
            records.forEach(item->{
                if (Objects.nonNull(finalLineIdMap) & Objects.nonNull(finalLineIdMap.get(item.getAirlineFileId()))){
                    item.setAirlineFileName( finalLineIdMap.get(item.getAirlineFileId()).getFileName() );
                }
                String vvx = item.getVvx();
                if (Objects.nonNull(vvx)){
                    String[] split = vvx.split(",");
                    LinkedHashMap<String,String> map = new LinkedHashMap<>();
                    for (int i = 0; i < split.length; i++) {
                        String[] newArray = split[i].split(":");
                        map.put(newArray[0], newArray[1]);
                    }
                    item.setVvMap(map);
                    String msg  = "流速数据" + map;
                    log.info(msg);
                }
            });
        }
        return JsonResult.success(airCurrentIPage);
    }


    /**
     * 硬件回传数据
     *
     * @param json
     * @return
     */
    @Override
    public Map getAirCurrent(String json){
        //Gson gson = new Gson();
        log.info("水测平台推送数据:" + json);
        if (Objects.isNull(json)){
            String msg  = "传输的json数据为空:" + json;
            log.info(msg);
        }
        //json转化为 对象
        AirCurrent airCurrent = new AirCurrent();
        //处理 每个时间段的数据
        VVX(json, airCurrent);
        //去查询最近的一条任务记录 取任务id 机场id 航线id
        List<InspectionRecord> inspectionRecordList = iInspectionRecordService.getFirstRecord();
        if (!CollectionUtils.isEmpty(inspectionRecordList)){
            InspectionRecord inspectionRecord = inspectionRecordList.get(0);
            //飞行记录id
            airCurrent.setInspectionRecordId(inspectionRecord.getId());
            //机场id
            airCurrent.setAirportId(inspectionRecord.getAid());
            //航线id
            airCurrent.setAirlineFileId(inspectionRecord.getAirlineFileId());
        }
        int insert = airCurrentMapper.insert(airCurrent);

        //返回第三方平台 数据已接受
        Map<String, Object> map = new HashMap<>();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String format = sdf.format(new Date());
        log.info("水测平台推送数据接受成功:" + json);
        map.put("TM", format);
        map.put("ST", airCurrent.getSt());
        map.put("ST", airCurrent.getSt());
        map.put("MARK", 1);
        map.put("SERIAL", airCurrent.getSerial());
        log.info("水测平台推送数据接受成功返回数据:" + map);
        return map;
    }


    @SneakyThrows
    private void VVX(String json,AirCurrent airCurrent) {
        log.info("开始处理水测平台推送数据:" + json);
        //处理各个时间段的 流速
        ObjectMapper objectMapper = new ObjectMapper();
        JsonNode jsonNode = objectMapper.readTree(json);
        //数据采集时间
        String airFlyTimeString = jsonNode.get("TT").asText();
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        try {
            airCurrent.setAirFlyTime( dateFormat.parse(airFlyTimeString) );
        } catch (ParseException e) {
            log.info("时间类型转化异常:" + airFlyTimeString);
            throw new RuntimeException(e);
        }
        //测站唯一编码 用于区分不同测站
        String st = Objects.isNull(jsonNode.get("ST")) ? null : jsonNode.get("ST").asText();
        airCurrent.setSt(st);
        //测站类型
        String type = Objects.isNull(jsonNode.get("TYPE")) ? null : jsonNode.get("TYPE").asText();
        airCurrent.setType(type);
        //数据包流水号
        Integer serial = Objects.isNull(jsonNode.get("SERIAL")) ? null : jsonNode.get("SERIAL").asInt();
        airCurrent.setSerial(serial);
        //总瞬时流量
        Double q1 = Objects.isNull(jsonNode.get("Q1")) ? null : jsonNode.get("Q1").asDouble();
        airCurrent.setQ1(q1);
        //断面平均流速
        Double v1 = Objects.isNull(jsonNode.get("V1")) ? null : jsonNode.get("V1").asDouble();
        airCurrent.setV1(v1);
        //垂线数量
        Integer VCNT = Objects.isNull(jsonNode.get("VCNT")) ? null : jsonNode.get("VCNT").asInt();
        airCurrent.setVcnt(VCNT);
        if(null == VCNT || VCNT <= 0){
            String msg  = "传输的垂线数量 数据为空:" + VCNT;
            log.info(msg);
        }else {
            //处理流速的数据 硬件给过来的格式是 VV01~VV20
            for (int i = 1; i <= VCNT; i++) {
                String num = "VV" + String.valueOf(i);
                //判空 空说声
                String aDouble = Objects.isNull(jsonNode.get(num)) ? null : jsonNode.get(num).asText();
                if (Objects.isNull(airCurrent.getVvx())){
                    airCurrent.setVvx(num + ":" + aDouble );
                }else {
                    airCurrent.setVvx(airCurrent.getVvx() + "," + num + ":" + aDouble );
                }
            }
        }
    }


}
