package com.tuoheng.airportGzdp.service.impl;


import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.JSONReader;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tuoheng.airportGzdp.common.JsonResult;
import com.tuoheng.airportGzdp.entity.Airport;
import com.tuoheng.airportGzdp.entity.Drone;
import com.tuoheng.airportGzdp.entity.Tenant;
import com.tuoheng.airportGzdp.mapper.AirportMapper;
import com.tuoheng.airportGzdp.service.IAirportService;
import com.tuoheng.airportGzdp.service.IDroneService;
import com.tuoheng.airportGzdp.service.ITaskService;
import com.tuoheng.airportGzdp.service.ITenantService;
import com.tuoheng.airportGzdp.until.RedisUtils;
import com.tuoheng.airportGzdp.vo.AirportListVo;
import lombok.extern.slf4j.Slf4j;
import org.eclipse.paho.client.mqttv3.internal.wire.MqttReceivedMessage;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;


import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * <p>
 * 机场表 服务实现类
 * </p>
 *
 * @author 拓恒
 * @since 2023-10-26
 */
@Service
@Slf4j
public class AirportServiceImpl extends ServiceImpl<AirportMapper, Airport> implements IAirportService {

    @Autowired
    private RedisUtils redisUtils;
    @Autowired
    private ITenantService tenantService;
    @Autowired
    private IDroneService droneService;
    @Resource
    private IAirportService iAirportService;

    /**
     * 查询机场列表
     * @return
     */
    @Override
    public JsonResult getList(String tenantCode) {
        try {
            Tenant tenant = tenantService.selectByTenantCode(tenantCode);
            IPage<Airport> page = new Page<>(1, Integer.MAX_VALUE);
            IPage<Airport> pageList = this.page(page, new LambdaQueryWrapper<Airport>()
                    .eq(Airport::getTenantId, tenant.getId()).eq(Airport::getMark, 1));
            //查询在线机场
            //String airportOnline = redisUtils.get("mqttClient") == null ? "" : redisUtils.get("mqttClient").toString();
            pageList.convert(x -> {
                AirportListVo airportListVo = Convert.convert(AirportListVo.class, x);
                Drone drone = droneService.getById(x.getDroneId());
                if (ObjectUtil.isNotEmpty(drone)) {
                    airportListVo.setCameraUrl(drone.getCameraUrl());
                    //通过 airport_id查询机场表的 ground_station_url
                    Airport byId = iAirportService.getById(drone.getAirportId());
                    log.info("通过airport_id查询机场表的ground_station_url查询结果", drone.getAirportId());
                    if (Objects.nonNull(byId)){
                        airportListVo.setGroundStationUrl(byId.getGroundStationUrl());
                    }
                }
                //status
                /*airportListVo.setStatus(1);
                Map statusMap = taskService.getRunTaskByAirportId(x.getId());
                if (ObjectUtil.isNotEmpty(statusMap)){
                    airportListVo.setStatus(2);
                    airportListVo.setTaskMap(statusMap);
                }*/
                return airportListVo;
            });
            return JsonResult.success(pageList);
        }catch (Exception e){
            log.error("",e);
        }
        return JsonResult.error();
    }

    /**
     * 查询在线机场
     *
     * @param airportOnline
     * @param airportListVo
     * @return
     */
    private boolean getOnline(String airportOnline, AirportListVo airportListVo) {
        JSONObject mqObject = JSONObject.parseObject(airportOnline);
        //新版
        /*if (ObjectUtil.isNotEmpty(redisUtils.get(airportListVo.getCode() + "_online"))) {
            if (redisUtils.get(airportListVo.getCode() + "_online").toString().equals("online")) {
                return true;
            } else {
                return false;
            }
        }*/
        if (mqObject.get("code").toString().equals("0")) {
            List list = (List) mqObject.get("data");
            if (list.size() > 0) {
                for (Object ob : list) {
                    JSONObject obs = (JSONObject) ob;
                    if (airportListVo.getEdgeId().equals(obs.get("clientid").toString())) {
                        return true;
                    };
                }
            }
            return false;
        }
        return false;
    }


    /**
     * 获取气象数据
     * @param airportId
     * @return
     */
    public JsonResult getWather(int airportId){
        Map map = new HashMap();
        MqttReceivedMessage wth = null;
        MqttReceivedMessage droneTmp = null;
        Airport airport  = this.getById(airportId);
        try {
            if (!ObjectUtils.isEmpty(redisUtils.hget(airport.getEdgeId() + "_data", "/data/WTH"))) {
                JSONObject jsonObject = new JSONObject();
                if (redisUtils.hget(airport.getEdgeId() + "_data", "/data/WTH") instanceof MqttReceivedMessage) {
                    log.info("天气数据MqttReceivedMessage类型：{}");
                    wth = (MqttReceivedMessage) redisUtils.hget(airport.getEdgeId() + "_data", "/data/WTH");
                    log.info("天气数据{}",wth.getPayload());
                     jsonObject =  (JSONObject)JSON.parse(wth.getPayload());
                }else{
                    log.info("天气数据JSONObject类型：{}");
                    jsonObject = (JSONObject) redisUtils.hget(airport.getEdgeId() + "_data", "/data/WTH");
                    log.info("天气数据{}",jsonObject);
                }
                map.put("wth",jsonObject.get("parmNew"));
            }
            if (!ObjectUtils.isEmpty(redisUtils.hget(airport.getEdgeId() + "_data", "/data/DroneBMS"))) {
                JSONObject jsonObject = new JSONObject();
                if (redisUtils.hget(airport.getEdgeId() + "_data", "/data/DroneBMS") instanceof MqttReceivedMessage) {
                    log.info("电池数据MqttReceivedMessage类型：{}");
                    droneTmp = (MqttReceivedMessage) redisUtils.hget(airport.getEdgeId() + "_data", "/data/DroneBMS");
                    jsonObject =  (JSONObject)JSON.parse(droneTmp.getPayload());
                }else{
                    log.info("电池数据JSONObject类型：{}");
                    jsonObject = (JSONObject) redisUtils.hget(airport.getEdgeId() + "_data", "/data/DroneBMS");
                }
                map.put("droneTmp",jsonObject.get("parm"));
            }

        } catch (Exception e) {
            log.error("",e);
        }
        return JsonResult.success(map);
    }

}
