package com.tuoheng.airportGzdp.service.impl;

import cn.hutool.core.thread.ThreadUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tuoheng.airportGzdp.common.JsonResult;
import com.tuoheng.airportGzdp.config.WebSocket;
import com.tuoheng.airportGzdp.entity.Tenant;
import com.tuoheng.airportGzdp.entity.VedioInspectionRecord;
import com.tuoheng.airportGzdp.mapper.TaskMapper;
import com.tuoheng.airportGzdp.mapper.VedioDataMapper;
import com.tuoheng.airportGzdp.service.ITaskService;
import com.tuoheng.airportGzdp.service.ITenantService;
import com.tuoheng.airportGzdp.until.RedisUtils;
import com.tuoheng.airportGzdp.vo.TaskListVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 机场任务表 服务实现类
 * </p>
 *
 * @author 拓恒
 * @since 2023-10-26
 */
@Service
@Slf4j
public class TaskServiceImpl extends ServiceImpl<TaskMapper, TaskListVo> implements ITaskService {

    @Autowired
    private TaskMapper taskMapper;
    @Autowired
    private VedioDataMapper vedioDataMapper;
    @Autowired
    private ITenantService tenantService;
    @Autowired
    private WebSocket webSocket;
    @Autowired
    RedisUtils redisUtils;
    /**
     * 查询机场任务列表
     * @return
     */
    @Override
    public JsonResult getList(String tenantCode,int limit) {
        Tenant tenant = tenantService.selectByTenantCode(tenantCode);
        //已完成的任务
        List listRun =  taskMapper.getRunList(tenant.getId(),limit);
        if (listRun.size()>=limit){
            return JsonResult.success(listRun);
        }
        if (listRun.size()>0){
            limit=limit-listRun.size();
        }
        List listAdd = new ArrayList();
        listAdd.addAll(listRun);
        //待执行的任务
        List listPre =  taskMapper.getPreList(tenant.getId(),limit);
        listAdd.addAll(listPre);
        if (listPre.size()>=limit){
            return JsonResult.success(listAdd);
        }
        if (listPre.size()>0){
            limit = limit-listPre.size();
        }
        //已完成的任务
        List list =  taskMapper.getList(tenant.getId(),limit);
        list.stream().map(t -> {
            //查询回放视频
            TaskListVo taskListVo = (TaskListVo) t;
            List<VedioInspectionRecord> vedioInspectionRecords =  vedioDataMapper.selectList(new LambdaQueryWrapper<VedioInspectionRecord>().eq(VedioInspectionRecord::getRecordId,taskListVo.getRecordId()));
            taskListVo.setVedioCount(vedioInspectionRecords.size());
            taskListVo.setListVedio(vedioInspectionRecords);
            return taskListVo;
        }).collect(Collectors.toList());;
        if (list.size()>=limit){
            listAdd.addAll(list);
            return JsonResult.success(listAdd);
        }
        return JsonResult.success(listAdd);
    }

    /**
     * 查询任务统计
     * @param tenantCode
     * @return
     */
    @Override
    public JsonResult getCountList(String tenantCode) {
        Tenant tenant = tenantService.selectByTenantCode(tenantCode);
         List list =  taskMapper.getCountList(tenant.getId());;
        return JsonResult.success(list);
    }

    public Map getRunTaskByAirportId(int id){
        List<Map> list =  taskMapper.getRunTaskByAirportId(id);
        if (list.size()>0){
            return list.get(0);
        }
        return null;
    }
    @Async
    public void getListToWebsocket(String tenantCode, int i) {
        while (true) {
            if (ObjectUtil.isNotEmpty(redisUtils.hget("NXDP","taskSendStatus"))
            &&!redisUtils.hget("NXDP","taskSendStatus").toString().equals(Thread.currentThread().getName())){
                return;
            }
            JsonResult jsonResult = getList(tenantCode, i);
            if (jsonResult.getCode()==0) {
                JSONObject jsonObject = new JSONObject();
                jsonObject.put("preTasklist",jsonResult.getData());
                webSocket.sendAllMessage(JSONObject.toJSONString(jsonObject));
            }
            ThreadUtil.sleep(3000);
            redisUtils.hset("NXDP","taskSendStatus",Thread.currentThread().getName(),10);
        }
    }
}
